# -*- coding: utf-8 -*-
# PyMoneyStats - On-Line Budget Manager
#
# Copyright (C) 2012 Mark Hall
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
u"""

.. moduleauthor:: Mark Hall <mark.hall@mail.room3b.eu>
"""

from sqlalchemy import (Column, Integer, Unicode, Float, ForeignKey, Table, Date)
from sqlalchemy.exc import OperationalError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import (scoped_session, sessionmaker, relationship)
from zope.sqlalchemy import ZopeTransactionExtension

DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
Base = declarative_base()

DB_VERSION = None
"""The currently required database version."""

class DBUpgradeException(Exception):
    """The :class:`~pyquest.models.DBUpgradeException` is used to indicate that
    the database requires an upgrade before the Experiment Support System system
    can be used.
    """
    def __init__(self, current, required):
        self.current = current
        self.required = required
    
    def __repr__(self):
        return "DBUpgradeException('%s', '%s'" % (self.current, self.required)
    
    def __str__(self):
        return """A database upgrade is required.

You are currently running version '%s', but version '%s' is required. Please run
alembic -c config.ini upgrade to upgrade the database and then start the application
again.
""" % (self.current, self.required)
    
def check_database_version():
    """Checks that the current version of the database matches the version specified
    by :data:`~pyquest.models.DB_VERSION`.
    """
    if DB_VERSION:
        dbsession = DBSession()
        try:
            result = dbsession.query('version_num').\
                    from_statement('SELECT version_num FROM alembic_version WHERE version_num = :version_num').\
                    params(version_num=DB_VERSION).first()
            if not result:
                result = dbsession.query('version_num').from_statement('SELECT version_num FROM alembic_version').first()
                raise DBUpgradeException(result[0], DB_VERSION)
        except OperationalError:
            raise DBUpgradeException('no version-information found', DB_VERSION)

line_item_tag = Table('line_items_tags',
                      Base.metadata,
                      Column('line_items_id', Integer, ForeignKey('line_items.id'), primary_key=True),
                      Column('tags_id', Integer, ForeignKey('tags.id'), primary_key=True))

class LineItem(Base):
    __tablename__ = 'line_items'
    
    id = Column(Integer, primary_key=True)
    title = Column(Unicode, index=True)
    amount = Column(Float)
    category = Column(Unicode, index=True)
    date = Column(Date)
    direction = Column(Unicode)

    tags = relationship('Tag', secondary=line_item_tag, backref='line_items', order_by='Tag.tag')

class Tag(Base):
    __tablename__ = 'tags'
    
    id = Column(Integer, primary_key=True)
    tag = Column(Unicode, unique=True, index=True)
    