# -*- coding: utf-8 -*-
# PyMoneyStats - On-Line Budget Manager
#
# Copyright (C) 2012 Mark Hall
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
u"""

.. moduleauthor:: Mark Hall <mark.hall@mail.room3b.eu>
"""
import datetime
import formencode

from formencode import validators
from pyramid.view import view_config
from pywebtools import text
from pywebtools.renderer import render
from sqlalchemy import and_, desc, asc

from pymoneystats import helpers
from pymoneystats.models import (DBSession, LineItem)

class OverviewSchema(formencode.Schema):
    
    year = validators.Number(if_missing=None, if_empty=None)
    month = validators.Number(if_missing=None, if_empty=None, min_value=1, max_value=12)
    
class OverviewDataSchema(formencode.Schema):
    
    interval = validators.UnicodeString(not_empty=True)
    analysis_start = validators.DateConverter(not_empty=True, month_style='dd/mm/yyyy')
    analysis_end = validators.DateConverter(not_empty=True, month_style='dd/mm/yyyy')
    display = validators.UnicodeString(not_empty=True)
    
@view_config(route_name='analysis.index')
@render({'text/html': 'analysis/index.html'})
def index(request):
    dbsession = DBSession()
    current_month= None
    last_entry = dbsession.query(LineItem.date).order_by(desc(LineItem.date)).first()
    if last_entry:
        current_month = last_entry.date
    return {'current_year': current_month}

@view_config(route_name='analysis.overview')
@render({'text/html': 'analysis/overview.html'})
def overview(request):
    dbsession = DBSession()
    data_start = dbsession.query(LineItem.date).order_by(asc(LineItem.date)).first()
    data_end = dbsession.query(LineItem.date).order_by(desc(LineItem.date)).first()
    if data_start and data_end:
        analysis_start = None
        analysis_end = None
        params = OverviewSchema().to_python(request.params)
        if params['year']:
            if params['month']:
                analysis_start = datetime.datetime(params['year'], params['month'], 1)
                if params['month'] == 12:
                    analysis_end = analysis_start.replace(year=params['year'] + 1, month=1) - datetime.timedelta(1)
                else:
                    analysis_end = analysis_start.replace(month=params['month'] + 1) - datetime.timedelta(1)
            else:
                analysis_start = datetime.datetime(params['year'], 1, 1)
                analysis_end = analysis_start.replace(year=params['year'] + 1) - datetime.timedelta(1)
            month_range = [('', 'Whole year')] + [(unicode(m), datetime.datetime(data_end.date.year, m, 1).strftime(('%B'))) for m in range(1, 12)]
        else:
            month_range = []
        year_range = [(unicode(y), unicode(y)) for y in range(data_start.date.year, data_end.date.year + 1)]
        
        totals = {'in': 0, 'out': 0}
        if analysis_start and analysis_end:
            items = dbsession.query(LineItem).filter(and_(LineItem.date>=analysis_start,
                                                          LineItem.date<=analysis_end))
            for item in items:
                if item.direction in totals:
                    totals[item.direction] = totals[item.direction] + item.amount
                else:
                    totals[item.direction] = item.amount
        totals['total'] = totals['in'] - totals['out']
    else:
        year_range = []
        month_range = []
        totals = {}
    return {'totals': totals,
            'year_range': year_range,
            'month_range': month_range,
            'analysis_start': analysis_start,
            'analysis_end': analysis_end}

def overview_cash_flow(request, params, markers):
    if params['interval'] == 'day':
        columns = [{'text': 'Date'}, {'text': 'Total', 'class_': 'text-right'}]
        rows = []
    else:
        columns = []
        rows = []
    dbsession = DBSession()
    for idx in range(0, len(markers) - 1):
        total = 0
        for item in dbsession.query(LineItem).filter(and_(LineItem.date>=markers[idx],
                                                          LineItem.date<markers[idx + 1])):
            if item.direction == 'in':
                total = total + item.amount
            elif item.direction == 'out':
                total = total - item.amount
        col = {'text': '%s %.2f' % (helpers.config_setting(request, 'currency.symbol', default='£'), total)}
        if total > 0:
            col['class_'] = 'text-right amount-in'
        elif total < 0:
            col['class_'] = 'text-right amount-out'
        else:
            col['class_'] = 'text-right'
        if params['interval'] == 'month':
            columns.append({'text': markers[idx].strftime('%b'), 'class_': 'text-center'})
            rows.append([col])
        elif params['interval'] == 'day':
            rows.append([{'text': markers[idx].strftime('%B %d'), 'class_': 'text-left'}, col])
    if params['interval'] == 'month':
        rows = [[r[0] for r in rows]]
    return {'columns': columns,
            'rows': rows}

def overview_category(request, params, markers):
    dbsession = DBSession()
    categories = {}
    for item in dbsession.query(LineItem).filter(and_(LineItem.date>=params['analysis_start'],
                                                      LineItem.date<params['analysis_end'],
                                                      LineItem.direction==params['display'][9:])):
        if item.category in categories:
            categories[item.category] = categories[item.category] + item.amount
        else:
            categories[item.category] = item.amount
    items = categories.items()
    items.sort(key=lambda c: c[0])
    rows = []
    for key, total in items:
        rows.append([{'text': text.title(key)}, {'text': '%s %.2f' % (helpers.config_setting(request, 'currency.symbol', default='£'), total), 'class_': 'text-right'}])
    return {'columns': [{'text': 'Category'}, {'text': 'Total', 'class_': 'text-right'}],
            'rows': rows}

def overview_tags(request, params, markers):
    dbsession = DBSession()
    tags = {}
    for item in dbsession.query(LineItem).filter(and_(LineItem.date>=params['analysis_start'],
                                                      LineItem.date<params['analysis_end'],
                                                      LineItem.direction==params['display'][5:])):
        for tag in item.tags:
            if tag.tag in tags:
                tags[tag.tag] = tags[tag.tag] + item.amount
            else:
                tags[tag.tag] = item.amount
        if not item.tags:
            if '[Untagged]' in tags:
                tags['[Untagged]'] = tags['[Untagged]'] + item.amount
            else:
                tags['[Untagged]'] = item.amount
    items = tags.items()
    items.sort(key=lambda c: c[0])
    rows = []
    for key, total in items:
        rows.append([{'text': text.title(key)}, {'text': '%s %.2f' % (helpers.config_setting(request, 'currency.symbol', default='£'), total), 'class_': 'text-right'}])
    return {'columns': [{'text': 'Tag'}, {'text': 'Total', 'class_': 'text-right'}],
            'rows': rows}

@view_config(route_name='analysis.overview.data')
@render({'application/json': True})
def overview_data(request):
    params = OverviewDataSchema().to_python(request.params)
    params['analysis_end'] = params['analysis_end'] + datetime.timedelta(days=1)
    if params['interval'] == 'day':
        markers = [params['analysis_start']]
        current = markers[0]
        while current < params['analysis_end']:
            current = current + datetime.timedelta(days=1)
            markers.append(current)
    elif params['interval'] == 'month':
        markers = [params['analysis_start']]
        current = markers[0]
        while current < params['analysis_end']:
            if current.month == 12:
                current = current.replace(year=current.year + 1, month=1)
            else:
                current = current.replace(month=current.month + 1)
            markers.append(current)
    data = {'columns': [],
            'rows': []}
    if params['display'] == 'cash_flow':
        data = overview_cash_flow(request, params, markers)
    elif params['display'].startswith('category_'):
        data = overview_category(request, params, markers)
    elif params['display'].startswith('tags_'):
        data = overview_tags(request, params, markers)
    return data
